import greenfoot.Actor;
import greenfoot.GreenfootImage;
import greenfoot.World;

import java.awt.Color;
import java.util.Collection;
import java.util.List;

/**
 * class of Threadfoot
 * <p>
 * </p>
 * Counterpart of Greenfoot-class World. Working with Threadfoot you should use
 * always this class instead of class World.
 * <p>
 * </p>
 * Remark 1: If you override the methods "started" or "stopped" in a derived
 * class you have to call the ThreadWorld-methods "started" respectively
 * "stopped" within those methods!
 * <p>
 * </p>
 * Remark 2: Do not use method "setBackground". Use "setImage" instead.
 * (World.setBackground is declared final and could not be overridden.)
 * 
 * @author Dietrich Boles (Uni Oldenburg, Germany)
 * @version (2.0) 2010/12/08
 * 
 */
public abstract class ThreadWorld extends World {

	/**
	 * Construct a new world. The size of the world (in number of cells) and the
	 * size of each cell (in pixels) must be specified.
	 * 
	 * @param worldWidth
	 *            The width of the world (in cells).
	 * @param worldHeight
	 *            The height of the world (in cells).
	 * @param cellSize
	 *            Size of a cell in pixels.
	 */
	public ThreadWorld(int worldWidth, int worldHeight, int cellSize) {
		super(worldWidth, worldHeight, cellSize);
		Threadfoot.setResetted();
	}

	/**
	 * Construct a new world. The size of the world (in number of cells) and the
	 * size of each cell (in pixels) must be specified. This constructor allows
	 * the option of creating an unbounded world, which actors can move outside
	 * the boundaries of.
	 * 
	 * @param worldWidth
	 *            The width of the world (in cells).
	 * @param worldHeight
	 *            The height of the world (in cells).
	 * @param cellSize
	 *            Size of a cell in pixels.
	 * @param bounded
	 *            Should actors be restricted to the world boundary?
	 */
	public ThreadWorld(int worldWidth, int worldHeight, int cellSize,
			boolean bounded) {
		super(worldWidth, worldHeight, cellSize, bounded);
		Threadfoot.setResetted();
	}

	/**
	 * This method is called by the Greenfoot system when the execution has
	 * started. This method can be overridden to implement custom behaviour when
	 * the execution is started.
	 * <p>
	 * </p>
	 * If you override this method in a derived class you have to call this
	 * method within those methods!
	 * 
	 * @see greenfoot.World#started()
	 */
	@Override
	public void started() {
		Threadfoot.startThreads(this);
	}

	/**
	 * This method is called by the Greenfoot system when the execution has
	 * stopped. This method can be overridden to implement custom behaviour when
	 * the execution is stopped.
	 * <p>
	 * </p>
	 * If you override this method in a derived class you have to call this
	 * method within those methods!
	 * 
	 * @see greenfoot.World#stopped()
	 */
	@Override
	public void stopped() {
		Threadfoot.stopThreads();
	}

	/**
	 * meaningless in Threadfoot
	 * 
	 * @see greenfoot.World#act()
	 */
	@Override
	public final void act() {
		// meaningless in Threadfoot
	}

	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * methods of Greenfoot-class World
	 */

	/**
	 * Set a background image for the world. If the image size is larger than
	 * the world in pixels, it is clipped. If it is smaller than the world, it
	 * is tiled. A pattern showing the cells can easily be shown by setting a
	 * background image with a size equal to the cell size.
	 * <p>
	 * </p>
	 * Do not use method "setBackground". Use "setImage" instead.
	 * 
	 * @see #setBackground(String)
	 * @param image
	 *            The image to be shown
	 */
	public void setImage(GreenfootImage image) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setBackground(image);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set a background image for the world from an image file. Images of type
	 * 'jpeg', 'gif' and 'png' are supported. If the image size is larger than
	 * the world in pixels, it is clipped. A pattern showing the cells can
	 * easily be shown by setting a background image with a size equal to the
	 * cell size.
	 * <p>
	 * </p>
	 * Do not use method "setBackground". Use "setImage" instead.
	 * 
	 * @see #setBackground(GreenfootImage)
	 * @param filename
	 *            The file holding the image to be shown
	 * @throws IllegalArgumentException
	 *             If the image can not be loaded.
	 */
	public void setImage(String filename) throws IllegalArgumentException {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setBackground(new ThreadfootImage(filename));
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the world's background image. The image may be used to draw onto
	 * the world's background.
	 * 
	 * @return The background image
	 */
	@Override
	public GreenfootImage getBackground() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getBackground();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the color at the centre of the cell. To paint a color, you need to
	 * get the background image for the world and paint on that.
	 * 
	 * @param x
	 *            The x coordinate of the cell.
	 * @param y
	 *            The y coordinate of the cell.
	 * @see #getBackground()
	 * @throws IndexOutOfBoundsException
	 *             If the location is not within the world bounds. If there is
	 *             no background image at the location it will return
	 *             Color.WHITE.
	 */
	@Override
	public Color getColorAt(int x, int y) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getColorAt(x, y);
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the width of the world (in number of cells).
	 */
	@Override
	public int getWidth() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getWidth();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the height of the world (in number of cells).
	 */
	@Override
	public int getHeight() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getHeight();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the size of a cell (in pixels).
	 */
	@Override
	public int getCellSize() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getCellSize();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set the paint order of objects in the world. Paint order is specified by
	 * class: objects of one class will always be painted on top of objects of
	 * some other class. The order of objects of the same class cannot be
	 * specified. Objects of classes listed first in the parameter list will
	 * appear on top of all objects of classes listed later.
	 * <p>
	 * Objects of a class not explicitly specified effectively inherit the paint
	 * order from their superclass.
	 * <p>
	 * Objects of classes not listed will appear below the objects whose classes
	 * have been specified.
	 * 
	 * @param classes
	 *            The classes in desired paint order
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public void setPaintOrder(Class... classes) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setPaintOrder(classes);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set the act order of objects in the world. Act order is specified by
	 * class: objects of one class will always act before objects of some other
	 * class. The order of objects of the same class cannot be specified.
	 * Objects of classes listed first in the parameter list will act before any
	 * objects of classes listed later.
	 * <p>
	 * Objects of a class not explicitly specified inherit the act order from
	 * their superclass.
	 * <p>
	 * Objects of classes not listed will act after all objects whose classes
	 * have been specified.
	 * 
	 * @param classes
	 *            The classes in desired act order
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public void setActOrder(Class... classes) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setActOrder(classes);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Add an Actor to the world.
	 * 
	 * @param object
	 *            The new object to add.
	 * @param x
	 *            The x coordinate of the location where the object is added.
	 * @param y
	 *            The y coordinate of the location where the object is added.
	 */
	@Override
	public synchronized void addObject(Actor object, int x, int y) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.addObject(object, x, y);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Remove an object from the world.
	 * 
	 * @param object
	 *            the object to remove
	 */
	@Override
	public synchronized void removeObject(Actor object) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.removeObject(object);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Remove a list of objects from the world.
	 * 
	 * @param objects
	 *            A list of Actors to remove.
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public synchronized void removeObjects(Collection objects) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.removeObjects(objects);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Get all the objects in the world, or all the objects of a particular
	 * class.
	 * <p>
	 * If a class is specified as a parameter, only objects of that class (or
	 * its subclasses) will be returned.
	 * <p>
	 * 
	 * @param cls
	 *            Class of objects to look for ('null' will find all objects).
	 * 
	 * @return A list of objects.
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public List getObjects(Class cls) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getObjects(cls);
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Get the number of actors currently in the world.
	 * 
	 * @return The number of actors
	 */
	@Override
	public int numberOfObjects() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.numberOfObjects();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Repaints the world.
	 */
	@Override
	public void repaint() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.repaint();
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return all objects at a given cell.
	 * <p>
	 * 
	 * An object is defined to be at that cell if its graphical representation
	 * overlaps with the cell at any point.
	 * 
	 * @param x
	 *            X-coordinate of the cell to be checked.
	 * @param y
	 *            Y-coordinate of the cell to be checked.
	 * @param cls
	 *            Class of objects to look return ('null' will return all
	 *            objects).
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public List getObjectsAt(int x, int y, Class cls) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getObjectsAt(x, y, cls);
			}
		} finally {
			Threadfoot.unlock();
		}
	}
}
