import greenfoot.GreenfootImage;

import java.awt.Color;
import java.awt.Font;
import java.awt.Shape;
import java.awt.image.BufferedImage;

/**
 * class of Threadfoot
 * <p>
 * </p>
 * Counterpart of Greenfoot-class GreenfootImage. Working with Threadfoot you
 * should use always this class instead of class GreenfootImage.
 * 
 * @author Dietrich Boles (Uni Oldenburg, Germany)
 * @version (2.0) 2010/12/08
 * 
 */
public class ThreadfootImage extends GreenfootImage {

	/**
	 * Create an image from an image file. Supported file formats are JPEG, GIF
	 * and PNG.
	 * <p>
	 * 
	 * The file name may be an absolute path, a base name for a file located in
	 * the project directory.
	 * 
	 * @param filename
	 *            Typically the name of a file in the images directory in the
	 *            project directory.
	 * @throws IllegalArgumentException
	 *             If the image can not be loaded.
	 */
	public ThreadfootImage(String filename) throws IllegalArgumentException {
		super(filename);
	}

	/**
	 * Create an empty (transparent) image with the specified size.
	 * 
	 * @param width
	 *            The width of the image in pixels.
	 * @param height
	 *            The height of the image in pixels.
	 */
	public ThreadfootImage(int width, int height) {
		super(width, height);
	}

	/**
	 * Create a ThreadfootImage from another ThreadfootImage.
	 */
	public ThreadfootImage(ThreadfootImage image)
			throws IllegalArgumentException {
		super(image);
	}

	/**
	 * Returns the java.awt.image.BufferedImage that backs this GreenfootImage.
	 * Any changes to the returned image will be reflected in the
	 * GreenfootImage.
	 * 
	 * @return The java.awt.image.BufferedImage backing this GreenfootImage
	 * @since Greenfoot version 1.0.2
	 */
	@Override
	public BufferedImage getAwtImage() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getAwtImage();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the width of the image.
	 * 
	 * @return Width of the image.
	 */
	@Override
	public int getWidth() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getWidth();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the height of the image.
	 * 
	 * @return Height of the image.
	 */
	@Override
	public int getHeight() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getHeight();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Rotates this image around the center.
	 * 
	 * @param degrees
	 */
	@Override
	public void rotate(int degrees) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.rotate(degrees);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Scales this image to a new size.
	 * 
	 * @param width
	 *            Width of new image
	 * @param height
	 *            Height of new image
	 */
	@Override
	public void scale(int width, int height) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.scale(width, height);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Mirrors the image vertically (flip around the y-axis).
	 * 
	 */
	@Override
	public void mirrorVertically() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.mirrorVertically();
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Mirrors the image horizontally (flip around the x-axis).
	 * 
	 */
	@Override
	public void mirrorHorizontally() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.mirrorHorizontally();
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Fill the entire image with the current drawing dcolor.
	 * 
	 */
	@Override
	public void fill() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.fill();
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draws the given Image onto this image
	 * 
	 * @param image
	 *            The image to draw onto this one.
	 * @param x
	 *            x-coordinate for drawing the image.
	 * @param y
	 *            y-coordinate for drawing the image.
	 */
	@Override
	public void drawImage(GreenfootImage image, int x, int y) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawImage(image, x, y);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set the current font. This font will be used for subsequent text
	 * operations.
	 */
	@Override
	public void setFont(Font f) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setFont(f);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Get the current font.
	 */
	@Override
	public Font getFont() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getFont();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set the current drawing color. This color will be used for subsequent
	 * drawing operations.
	 * 
	 * @param color
	 *            The color to be used.
	 */
	@Override
	public void setColor(Color color) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setColor(color);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the current drawing color.
	 * 
	 * @return The current color.
	 */
	@Override
	public Color getColor() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getColor();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the color at the given pixel.
	 * 
	 * @throws IndexOutOfBoundsException
	 *             If the pixel location is not within the image bounds.
	 */
	@Override
	public Color getColorAt(int x, int y) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getColorAt(x, y);
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Sets the color at the given pixel to the given color.
	 */
	@Override
	public void setColorAt(int x, int y, Color color) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setColorAt(x, y, color);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Set the transparency of the image.
	 * 
	 * @param t
	 *            A value in the range 0 to 255. 0 is completely transparent
	 *            (invisible) and 255 is completely opaque (the default).
	 */
	@Override
	public void setTransparency(int t) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.setTransparency(t);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return the current transparency of the image.
	 * 
	 * @return A value in the range 0 to 255. 0 is completely transparent
	 *         (invisible) and 255 is completely opaque (the default).
	 */
	@Override
	public int getTransparency() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.getTransparency();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Fill the specified rectangle. The left and right edges of the rectangle
	 * are at <code>x</code> and <code>x&nbsp;+&nbsp;width&nbsp;-&nbsp;1</code>.
	 * The top and bottom edges are at <code>y</code> and
	 * <code>y&nbsp;+&nbsp;height&nbsp;-&nbsp;1</code>. The resulting rectangle
	 * covers an area <code>width</code> pixels wide by <code>height</code>
	 * pixels tall. The rectangle is filled using the current color.
	 * 
	 * @param x
	 *            the <i>x </i> coordinate of the rectangle to be filled.
	 * @param y
	 *            the <i>y </i> coordinate of the rectangle to be filled.
	 * @param width
	 *            the width of the rectangle to be filled.
	 * @param height
	 *            the height of the rectangle to be filled.
	 */
	@Override
	public void fillRect(int x, int y, int width, int height) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.fillRect(x, y, width, height);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Clears the image.
	 * 
	 */
	@Override
	public void clear() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.clear();
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw the outline of the specified rectangle. The left and right edges of
	 * the rectangle are at <code>x</code> and <code>x&nbsp;+&nbsp;width</code>.
	 * The top and bottom edges are at <code>y</code> and
	 * <code>y&nbsp;+&nbsp;height</code>. The rectangle is drawn using the
	 * current color.
	 * 
	 * @param x
	 *            the <i>x </i> coordinate of the rectangle to be drawn.
	 * @param y
	 *            the <i>y </i> coordinate of the rectangle to be drawn.
	 * @param width
	 *            the width of the rectangle to be drawn.
	 * @param height
	 *            the height of the rectangle to be drawn.
	 */
	@Override
	public void drawRect(int x, int y, int width, int height) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawRect(x, y, width, height);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw the text given by the specified string, using the current font and
	 * color. The baseline of the leftmost character is at position ( <i>x
	 * </i>,&nbsp; <i>y </i>).
	 * 
	 * @param string
	 *            the string to be drawn.
	 * @param x
	 *            the <i>x </i> coordinate.
	 * @param y
	 *            the <i>y </i> coordinate.
	 */
	@Override
	public void drawString(String string, int x, int y) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawString(string, x, y);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw a shape directly on the image. Shapes are specified by the <a href=
	 * "http://java.sun.com/javase/6/docs/api/java/awt/Shape.html">shape
	 * interface</a>.
	 * 
	 * @param shape
	 *            the shape to be drawn.
	 */
	@Override
	public void drawShape(Shape shape) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawShape(shape);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw a filled shape directly on the image. Shapes are specified by the <a
	 * href="http://java.sun.com/javase/6/docs/api/java/awt/Shape.html">shape
	 * interface</a>.
	 * 
	 * @param shape
	 *            the shape to be drawn.
	 */
	@Override
	public void fillShape(Shape shape) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.fillShape(shape);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Fill an oval bounded by the specified rectangle with the current drawing
	 * color.
	 * 
	 * @param x
	 *            the <i>x </i> coordinate of the upper left corner of the oval
	 *            to be filled.
	 * @param y
	 *            the <i>y </i> coordinate of the upper left corner of the oval
	 *            to be filled.
	 * @param width
	 *            the width of the oval to be filled.
	 * @param height
	 *            the height of the oval to be filled.
	 */
	@Override
	public void fillOval(int x, int y, int width, int height) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.fillOval(x, y, width, height);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw an oval bounded by the specified rectangle with the current drawing
	 * color.
	 * 
	 * @param x
	 *            the <i>x </i> coordinate of the upper left corner of the oval
	 *            to be filled.
	 * @param y
	 *            the <i>y </i> coordinate of the upper left corner of the oval
	 *            to be filled.
	 * @param width
	 *            the width of the oval to be filled.
	 * @param height
	 *            the height of the oval to be filled.
	 */
	@Override
	public void drawOval(int x, int y, int width, int height) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawOval(x, y, width, height);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Fill a closed polygon defined by arrays of <i>x </i> and <i>y </i>
	 * coordinates.
	 * <p>
	 * This method draws the polygon defined by <code>nPoint</code> line
	 * segments, where the first <code>nPoint&nbsp;-&nbsp;1</code> line segments
	 * are line segments from
	 * <code>(xPoints[i&nbsp;-&nbsp;1],&nbsp;yPoints[i&nbsp;-&nbsp;1])</code> to
	 * <code>(xPoints[i],&nbsp;yPoints[i])</code>, for 1&nbsp;&le;&nbsp; <i>i
	 * </i>&nbsp;&le;&nbsp; <code>nPoints</code>. The figure is automatically
	 * closed by drawing a line connecting the final point to the first point,
	 * if those points are different.
	 * <p>
	 * The area inside the polygon is defined using an even-odd fill rule, also
	 * known as the alternating rule.
	 * 
	 * @param xPoints
	 *            a an array of <code>x</code> coordinates.
	 * @param yPoints
	 *            a an array of <code>y</code> coordinates.
	 * @param nPoints
	 *            a the total number of points.
	 */
	@Override
	public void fillPolygon(int[] xPoints, int[] yPoints, int nPoints) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.fillPolygon(xPoints, yPoints, nPoints);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draws a closed polygon defined by arrays of <i>x</i> and <i>y</i>
	 * coordinates. Each pair of (<i>x</i>,&nbsp;<i>y</i>) coordinates defines a
	 * point.
	 * <p>
	 * This method draws the polygon defined by <code>nPoint</code> line
	 * segments, where the first <code>nPoint&nbsp;-&nbsp;1</code> line segments
	 * are line segments from
	 * <code>(xPoints[i&nbsp;-&nbsp;1],&nbsp;yPoints[i&nbsp;-&nbsp;1])</code> to
	 * <code>(xPoints[i],&nbsp;yPoints[i])</code>, for
	 * 1&nbsp;&le;&nbsp;<i>i</i>&nbsp;&le;&nbsp;<code>nPoints</code>. The figure
	 * is automatically closed by drawing a line connecting the final point to
	 * the first point, if those points are different.
	 * 
	 * @param xPoints
	 *            an array of <code>x</code> coordinates.
	 * @param yPoints
	 *            an array of <code>y</code> coordinates.
	 * @param nPoints
	 *            the total number of points.
	 */
	@Override
	public void drawPolygon(int[] xPoints, int[] yPoints, int nPoints) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawPolygon(xPoints, yPoints, nPoints);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Draw a line, using the current drawing color, between the points
	 * <code>(x1,&nbsp;y1)</code> and <code>(x2,&nbsp;y2)</code>.
	 * 
	 * @param x1
	 *            the first point's <i>x </i> coordinate.
	 * @param y1
	 *            the first point's <i>y </i> coordinate.
	 * @param x2
	 *            the second point's <i>x </i> coordinate.
	 * @param y2
	 *            the second point's <i>y </i> coordinate.
	 */
	@Override
	public void drawLine(int x1, int y1, int x2, int y2) {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.drawLine(x1, y1, x2, y2);
			}
			Threadfoot.repaintWorld();
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Return a text representation of the image.
	 */
	@Override
	public String toString() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.toString();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

}