import greenfoot.GreenfootSound;

/**
 * class of Threadfoot
 * <p>
 * </p>
 * Counterpart of Greenfoot-class GreenfootSound. Working with Threadfoot you
 * should use always this class instead of class GreenfootSound.
 * 
 * @author Dietrich Boles (Uni Oldenburg, Germany)
 * @version (2.0) 2010/12/08
 * 
 */
public class ThreadfootSound extends GreenfootSound {

	/**
	 * Creates a new sound from the given file.
	 * 
	 * @param filename
	 *            Typically the name of a file in the sounds directory in the
	 *            project directory.
	 */
	public ThreadfootSound(String filename) {
		super(filename);
	}

	/**
	 * Start playing this sound. If it is playing already, it will do nothing.
	 * If the sound is currently looping, it will finish the current loop and
	 * stop. If the sound is currently paused, it will resume playback from the
	 * point where it was paused. The sound will be played once.
	 */
	@Override
	public void play() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.play();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Play this sound repeatedly in a loop. If called on an already looping
	 * sound, it will do nothing. If the sound is already playing once, it will
	 * start looping instead. If the sound is currently paused, it will resume
	 * playing from the point where it was paused.
	 */
	@Override
	public void playLoop() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.playLoop();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Stop playing this sound if it is currently playing. If the sound is
	 * played again later, it will start playing from the beginning. If the
	 * sound is currently paused it will now be stopped instead.
	 */
	@Override
	public void stop() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.stop();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Pauses the current sound if it is currently playing. If the sound is
	 * played again later, it will resume from the point where it was paused.
	 * <p>
	 * </p>
	 * Make sure that this is really the method you want. If possible, you
	 * should always use stop(), because the resources can be released after
	 * calling stop(). The resources for the sound will not be released while it
	 * is paused.
	 */
	@Override
	public void pause() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				super.pause();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * True if the sound is currently playing.
	 */
	@Override
	public boolean isPlaying() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.isPlaying();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

	/**
	 * Returns a string representation of this sound containing the name of the
	 * file and whether it is currently playing or not.
	 */
	@Override
	public String toString() {
		Threadfoot.lock();
		try {
			synchronized (Threadfoot.getSyncObject()) {
				return super.toString();
			}
		} finally {
			Threadfoot.unlock();
		}
	}

}